#!/usr/bin/env node
const program = require('commander')
const path = require('path')
const fs = require('fs')
const exec = require('child_process').exec
const inquirer = require('inquirer')
const shell = require('shelljs')
const glob = require('glob')
const ora = require('ora')
const colors = require('../lib/colors')
const generator = require('../templates/generator')
const download = require('../lib/download')

program.usage('<project-name>').parse(process.argv)

// 判断文件夹不存在


// 运行环境path
let processPath = process.cwd()
// 根据输入，获取项目名称
let projectName = program.args[0]

// 遍历当前目录
const list = glob.sync('*')
let next = undefined
let rootName = path.basename(process.cwd())
// 如果当前目录不为空
new Promise((resolve,reject) => {
	if (list.length) {
		if (list.filter(name => {
				const fileName = path.resolve(processPath, path.join('.', name))
				const isDir = fs.lstatSync(fileName).isDirectory()
				return name.indexOf(projectName) !== -1 && isDir
			}).length !== 0) {
			console.log(`项目${projectName}已经存在`.error)
			return
		}
		resolve(projectName)
	} else if (rootName === projectName) {
		next = inquirer.prompt([
			{
				name: 'buildInCurrent',
				message: '当前目录为空，且目录名称和项目名称相同，是否直接在当前目录下创建新项目？',
				type: 'confirm',
				default: true
			}
		]).then(answer => {
			resolve(answer.buildInCurrent ? '.' : projectName)
		})
	} else {
		resolve(projectName)
	}
}).then(projectRoot => {
	if (projectRoot !== '.') {
				fs.mkdir(projectRoot, () => {
			resolve()
		})
	}
	getProjetInfo()
})

// new Promise((resolve, reject) => {
// 	// 判断目标文件夹是否存在
// 	if (!fs.existsSync(rootPath)) {
// 		fs.mkdir(rootPath, () => {
// 			resolve()
// 		})
// 	} else {
// 		if (!dirIsNull(rootPath)) {
// 			program.help()
// 			console.log(`文件夹非空：${processPath}`.warn)
// 		}
//
// 		inquirer
// 			.prompt([{
// 				name: 'init',
// 				message: `确定在 ${rootPath} 目录下生成项目？ Y/N`,
// 				default: 'Y'
// 			}
// 			])
// 			.then(answers => {
// 				if (answers.init.toUpperCase() == 'Y') {
// 					resolve()
// 				} else {
// 					console.log('初始化项目失败'.error)
// 				}
// 			})
// 	}
// }).then(() => {
// 	getProjetInfo()
// })

function getProjetInfo () {
	console.log('开始构建项目'.info)
	
	download(rootPath)
		.then(target => {
			console.log(`======${target}=======`)
			
			let context = {
				name: projectName || '项目名称'
			}
			inquirer
				.prompt([
					{
						name: 'projectName',
						message: '项目的名称',
						default: context.name
					}, {
						name: 'projectVersion',
						message: '项目的版本号',
						default: '1.0.0'
					}, {
						name: 'projectDescription',
						message: '项目的简介',
						default: `A project named `
					}, {
						name: 'projectauthor',
						message: '项目的作者',
						default: `A project created by...`
					}
				])
				.then(answers => {
					// 用户输入自定义项赋值给context传入package.json
					context = answers
					generator(context, rootPath)
					shell.rm('-rf', path.resolve(rootPath, '.git'))
					console.log(`项目构建成功:${rootPath}`.info)
					choosePackageSource()
				})
		})
		.catch(err => console.log(err.error))
}

function choosePackageSource () {
	let npmChoice = ['使用 yarn 安装', '使用 cnpm 安装', '使用 npm 安装', '稍后自己安装']
	inquirer
		.prompt([
			{
				type: 'list',
				name: 'npm',
				message: '安装npm依赖包?',
				choices: npmChoice,
				default: 0
			}
		])
		.then(answers => {
			let npmIndex = npmChoice.indexOf(answers.npm)
			let shellStr = ''
			switch (npmIndex) {
				case 0: shellStr = 'yarn install'
					break
				case 1: shellStr = 'cnpm install'
					break
				case 2: shellStr = 'npm install'
					break
			}
			const spinner = ora(`正在安装NPM依赖`)
			spinner.start()
			
			if (shellStr != '') {
				exec(shellStr, (err, stdout, stderr) => {
					if (err) {
						spinner.fail()
						console.log(stderr.error)
					} else {
						spinner.succeed()
						console.log('NPM安装完成'.info)
					}
				})
			}
		})
}
